// ======================================================================
//
// MenuVerticalSliderBar.cpp
// hugh david & ~Chuck Karpiak
//
// copyright 1997 origin systems inc
//
// ======================================================================

#include "First.h"
#include "MenuVerticalSliderBar.h"

#include "Bitmap.h"
#include "Iff.h"
#include "Mouse.h"

// ======================================================================

const Tag TAG_BITS = TAG(B,I,T,S);
const Tag TAG_VSBE = TAG(V,S,B,E);

// ======================================================================

MenuVerticalSliderBar::MenuVerticalSliderBar(MenuElement *newParent, int maxWidth, int maxHeight, Iff *iff)
: MenuElement(newParent, maxWidth, maxHeight, NULL)
{
	percentage        = 0.0f;
	yOffset           = 0;
	eventNumber       = 0;
	sliderBarHeight   = 0;
	sliderBarYHotSpot = 0;
	bitmap            = NULL;
	pressingLeftMouse = false;

	load(iff);
}

// ----------------------------------------------------------------------

MenuVerticalSliderBar::~MenuVerticalSliderBar(void)
{
}

// ----------------------------------------------------------------------

void MenuVerticalSliderBar::load(Iff *iff)
{
	char buffer[128];
	int  x;

	if (iff)
	{
		MenuElement::load(iff);

		iff->seekChunk(TAG_BITS);
		iff->read_string(buffer, sizeof(buffer));
		if ((bitmap = findSharedBitmap(buffer)) == NULL)
			Fatal("MenuVerticalSliderBar::load !findSharedBitmap '%s'", buffer);
		if (bitmap->frames() < 2)
			Fatal("MenuVerticalSliderBar::load shared bitmap '%s' only contains %d frames (need >= 2)", buffer, bitmap->frames());
		iff->exitChunk();

		// record off the slider bar's height for use in computing yOffset
		sliderBarHeight   = bitmap->height(1);
		bitmap->getHotspot(&x, &sliderBarYHotSpot);

		iff->seekChunk(TAG_VSBE);
		eventNumber = iff->read_int32();
		iff->exitChunk();
	}

	// initialize the yOffset
	clampYOffset();
}

// ----------------------------------------------------------------------
// hover event messages occur only once for each 'start' and 'stop'
IoResult MenuVerticalSliderBar::event(IoEventStruct *event)
{
	static IoWin* old_capturer = NULL;
	Rect          r;

	r = menuRect;
	if (pressingLeftMouse)
		getAbsoluteRect(&r);

	if (r.isInInclusive(event->mouseX, event->mouseY))
	{
		// start dragging?
		if (event->type == IoEventMousePress && event->button == 0)
		{
			pressingLeftMouse = true;
			old_capturer = IoWinManager::capture (this);
			return IoResultBlock;
		}

		// end dragging?
		if (event->type == IoEventMouseRelease && event->button == 0)
		{
			pressingLeftMouse = false;
			// stop capturing
			IoWinManager::release (this, old_capturer);
			return IoResultBlock;
		}
	}
	else
		if (event->type == IoEventMouseRelease && event->button == 0)
		{
			pressingLeftMouse = false;
			// stop capturing
			IoWinManager::release (this, old_capturer);
			return IoResultBlock;
		}

	// are we currently dragging?
	if (pressingLeftMouse && event->type == IoEventMouseSetCursor)
	{
		calculatePercentage(menuRect.y0 + event->mouseY - r.y0);
		calculateYOffset();
		MenuElement::menuEvent(eventNumber, this);
		return IoResultBlock;
	}

	if (pressingLeftMouse)
		return IoResultBlock;

	return MenuElement::event(event);
}

// ----------------------------------------------------------------------

void MenuVerticalSliderBar::menuDraw(int x, int y) const
{
	MenuElement::menuDraw(x, y);

	bitmap->draw(x + menuRect.x0, y + menuRect.y0, 0);
	bitmap->draw(x + menuRect.x0, y + menuRect.y0 + yOffset, 1);

//	GL::line(x + menuRect.x0 - 10, y + menuRect.y0 + sliderBarYHotSpot, x + menuRect.x1 + 10, y + menuRect.y0 + sliderBarYHotSpot, GL::green());
//	GL::line(x + menuRect.x0 - 10, y + menuRect.y1 - (sliderBarHeight - sliderBarYHotSpot), x + menuRect.x1 + 10, y + menuRect.y1 - (sliderBarHeight - sliderBarYHotSpot), GL::green());
//
//	GL::line(x + menuRect.x0 - 10, y + menuRect.y0+ yOffset, x + menuRect.x1 + 10, y + menuRect.y0 + yOffset, GL::white());
}

// ----------------------------------------------------------------------

void MenuVerticalSliderBar::calculatePercentage(int mouseY)
{
	percentage =  static_cast<real>(mouseY - menuRect.y0 - sliderBarYHotSpot);
	percentage /= menuRect.y1 - menuRect.y0 - (sliderBarHeight);

//DPRINT_LOG(("percentage before = %f\n", percentage));

	// clamp it between 0.0f and 1.0f
	if (percentage < CONST_REAL(0))
		percentage = CONST_REAL(0);
	if (percentage > CONST_REAL(1))
		percentage = CONST_REAL(1);
}

// ----------------------------------------------------------------------
// assumes that curValue has been updated
void MenuVerticalSliderBar::calculateYOffset(void)
{
	// set the slider drawing offset
	yOffset = static_cast<int>(percentage * (menuRect.y1 - menuRect.y0 - (sliderBarHeight))) + sliderBarYHotSpot;
	clampYOffset();
}

// ----------------------------------------------------------------------
// clamp down the yOffset to include the slider bar's height
inline void MenuVerticalSliderBar::clampYOffset(void)
{
	// clamp down the yOffset to include the slider bar's height
	if (yOffset < sliderBarYHotSpot)
		yOffset = sliderBarYHotSpot;
	if (yOffset > (menuRect.y1 - (sliderBarHeight - sliderBarYHotSpot - 1)) - menuRect.y0)
		yOffset = (menuRect.y1 - (sliderBarHeight - sliderBarYHotSpot - 1)) - menuRect.y0;
}

// ----------------------------------------------------------------------

void MenuVerticalSliderBar::setPercentage(real newPercentage)
{
	percentage = newPercentage;
	if (percentage < CONST_REAL(0))
		percentage = CONST_REAL(0);
	if (percentage > CONST_REAL(1))
		percentage = CONST_REAL(1);

	calculateYOffset();
}

// ----------------------------------------------------------------------
